@@ -1,5 +1,5 @@
 /*
- * eval.c
+ * expression.c
  *
  *  Created on: Apr 24, 2017
  *      Author: skauffma
@@ -60,7 +60,7 @@ void evaluate_expression(expression_input *input, typed_value *result, data_stac
     unsigned int position, length;
     expression_action action;
     map_key key;
-    stack_value value, left_value, right_value;
+    stack_value typed, left_value, right_value;
 
     // length is stored as the first element in the action list
     length = input[0].length;
@@ -75,125 +75,125 @@ void evaluate_expression(expression_input *input, typed_value *result, data_stac
             pop(stack, &right_value);
             pop(stack, &left_value);
             if (left_value.type == real_type) {
-                value.type = real_type;
+                typed.type = real_type;
 
                 if (right_value.type == real_type) {
-                    value.real_value = left_value.real_value + right_value.real_value;
+                    typed.value.real = left_value.value.real + right_value.value.real;
                 } else if (right_value.type == integer_type) {
-                    value.real_value = left_value.real_value + right_value.integer_value;
+                    typed.value.real = left_value.value.real + right_value.value.integer;
                 } else {
                     filter_log_msg(LOG_LEVEL_ERROR, "Type error in expression evaluation: non-numeric value on right side of add\n");
                 }
             } else if (left_value.type == integer_type) {
                 if (right_value.type == real_type) {
-                    value.type = real_type;
-                    value.real_value = left_value.integer_value + right_value.real_value;
+                    typed.type = real_type;
+                    typed.value.real = left_value.value.integer + right_value.value.real;
                 } else if (right_value.type == integer_type) {
-                    value.type = integer_type;
-                    value.integer_value = left_value.integer_value + right_value.integer_value;
+                    typed.type = integer_type;
+                    typed.value.integer = left_value.value.integer + right_value.value.integer;
                 } else {
                     filter_log_msg(LOG_LEVEL_ERROR, "Type error in expression evaluation: non-numeric value on right side of add\n");
                 }
             } else {
                 filter_log_msg(LOG_LEVEL_ERROR, "Type error in expression evaluation: non-numeric value on left side of add\n");
             }
-            push(stack, &value);
+            push(stack, &typed);
             break;
         case action_sub:
             pop(stack, &right_value);
             pop(stack, &left_value);
             if (left_value.type == real_type) {
-                value.type = real_type;
+                typed.type = real_type;
 
                 if (right_value.type == real_type) {
-                    value.real_value = left_value.real_value - right_value.real_value;
+                    typed.value.real = left_value.value.real - right_value.value.real;
                 } else if (right_value.type == integer_type) {
-                    value.real_value = left_value.real_value - right_value.integer_value;
+                    typed.value.real = left_value.value.real - right_value.value.integer;
                 } else {
                     filter_log_msg(LOG_LEVEL_ERROR, "Type error in expression evaluation: non-numeric value on right side of sub\n");
                 }
             } else if (left_value.type == integer_type) {
                 if (right_value.type == real_type) {
-                    value.type = real_type;
-                    value.real_value = left_value.integer_value - right_value.real_value;
+                    typed.type = real_type;
+                    typed.value.real = left_value.value.integer - right_value.value.real;
                 } else if (right_value.type == integer_type) {
-                    value.type = integer_type;
-                    value.integer_value = left_value.integer_value - right_value.integer_value;
+                    typed.type = integer_type;
+                    typed.value.integer = left_value.value.integer - right_value.value.integer;
                 } else {
                     filter_log_msg(LOG_LEVEL_ERROR, "Type error in expression evaluation: non-numeric value on right side of sub\n");
                 }
             } else {
                 filter_log_msg(LOG_LEVEL_ERROR, "Type error in expression evaluation: non-numeric value on left side of sub\n");
             }
-            push(stack, &value);
+            push(stack, &typed);
             break;
         case action_mul:
             pop(stack, &right_value);
             pop(stack, &left_value);
             if (left_value.type == real_type) {
-                value.type = real_type;
+                typed.type = real_type;
 
                 if (right_value.type == real_type) {
-                    value.real_value = left_value.real_value * right_value.real_value;
+                    typed.value.real = left_value.value.real * right_value.value.real;
                 } else if (right_value.type == integer_type) {
-                    value.real_value = left_value.real_value * right_value.integer_value;
+                    typed.value.real = left_value.value.real * right_value.value.integer;
                 } else {
                     filter_log_msg(LOG_LEVEL_ERROR, "Type error in expression evaluation: non-numeric value on right side of mul\n");
                 }
             } else if (left_value.type == integer_type) {
                 if (right_value.type == real_type) {
-                    value.type = real_type;
-                    value.real_value = left_value.integer_value * right_value.real_value;
+                    typed.type = real_type;
+                    typed.value.real = left_value.value.integer * right_value.value.real;
                 } else if (right_value.type == integer_type) {
-                    value.type = integer_type;
-                    value.integer_value = left_value.integer_value * right_value.integer_value;
+                    typed.type = integer_type;
+                    typed.value.integer = left_value.value.integer * right_value.value.integer;
                 } else {
                     filter_log_msg(LOG_LEVEL_ERROR, "Type error in expression evaluation: non-numeric value on right side of mul\n");
                 }
             } else {
                 filter_log_msg(LOG_LEVEL_ERROR, "Type error in expression evaluation: non-numeric value on left side of mul\n");
             }
-            push(stack, &value);
+            push(stack, &typed);
             break;
         case action_div:
             pop(stack, &right_value);
             pop(stack, &left_value);
             if (left_value.type == real_type) {
-                value.type = real_type;
+                typed.type = real_type;
 
                 if (right_value.type == real_type) {
-                    if (right_value.real_value != 0.0) {
-                        value.real_value = left_value.real_value / right_value.real_value;
+                    if (right_value.value.real != 0.0) {
+                        typed.value.real = left_value.value.real / right_value.value.real;
                     } else {
                         filter_log_msg(LOG_LEVEL_WARN, "Divide by zero error in expression evaluation\n");
-                        value.real_value = 0.0;
+                        typed.value.real = 0.0;
                     }
                 } else if (right_value.type == integer_type) {
-                    if (right_value.integer_value != 0) {
-                        value.real_value = left_value.real_value / right_value.integer_value;
+                    if (right_value.value.integer != 0) {
+                        typed.value.real = left_value.value.real / right_value.value.integer;
                     } else {
                         filter_log_msg(LOG_LEVEL_WARN, "Divide by zero error in expression evaluation\n");
-                        value.real_value = 0.0;
+                        typed.value.real = 0.0;
                     }
                 } else {
                     filter_log_msg(LOG_LEVEL_ERROR, "Type error in expression evaluation: non-numeric value on right side of div\n");
                 }
             } else if (left_value.type == integer_type) {
                 if (right_value.type == real_type) {
-                    value.type = real_type;
-                    if (right_value.real_value != 0.0) {
-                        value.real_value = left_value.integer_value / right_value.real_value;
+                    typed.type = real_type;
+                    if (right_value.value.real != 0.0) {
+                        typed.value.real = left_value.value.integer / right_value.value.real;
                     } else {
                         filter_log_msg(LOG_LEVEL_WARN, "Divide by zero error in expression evaluation\n");
-                        value.real_value = 0.0;
+                        typed.value.real = 0.0;
                     }
                 } else if (right_value.type == integer_type) {
-                    value.type = integer_type;
-                    if (right_value.integer_value != 0) {
-                        value.integer_value = left_value.integer_value / right_value.integer_value;
+                    typed.type = integer_type;
+                    if (right_value.value.integer != 0) {
+                        typed.value.integer = left_value.value.integer / right_value.value.integer;
                     } else {
                         filter_log_msg(LOG_LEVEL_WARN, "Divide by zero error in expression evaluation\n");
-                        value.integer_value = 0.0;
+                        typed.value.integer = 0.0;
                     }
                 } else {
                     filter_log_msg(LOG_LEVEL_ERROR, "Type error in expression evaluation: non-numeric value on right side of div\n");
@@ -201,19 +201,19 @@ void evaluate_expression(expression_input *input, typed_value *result, data_stac
             } else {
                 filter_log_msg(LOG_LEVEL_ERROR, "Type error in expression evaluation: non-numeric value on left side of div\n");
             }
-            push(stack, &value);
+            push(stack, &typed);
             break;
         case action_mod:
             pop(stack, &right_value);
             pop(stack, &left_value);
             if (left_value.type == integer_type) {
                 if (right_value.type == integer_type) {
-                    value.type = integer_type;
-                    if (right_value.integer_value != 0) {
-                        value.integer_value = left_value.integer_value % right_value.integer_value;
+                    typed.type = integer_type;
+                    if (right_value.value.integer != 0) {
+                        typed.value.integer = left_value.value.integer % right_value.value.integer;
                     } else {
                         filter_log_msg(LOG_LEVEL_WARN, "Divide by zero error in expression evaluation\n");
-                        value.integer_value = 0.0;
+                        typed.value.integer = 0.0;
                     }
                 } else {
                     filter_log_msg(LOG_LEVEL_ERROR, "Type error in expression evaluation: non-integer value on right side of mod\n");
@@ -221,157 +221,157 @@ void evaluate_expression(expression_input *input, typed_value *result, data_stac
             } else {
                 filter_log_msg(LOG_LEVEL_ERROR, "Type error in expression evaluation: non-integer value on left side of mod\n");
             }
-            push(stack, &value);
+            push(stack, &typed);
             break;
 
         case action_lt:
             pop(stack, &right_value);
             pop(stack, &left_value);
-            value.type = boolean_type;
+            typed.type = boolean_type;
             // default to false if there is an error
-            value.boolean_value = false;
+            typed.value.boolean = false;
 
             if (left_value.type == real_type) {
                 if (right_value.type == real_type) {
-                    value.boolean_value = left_value.real_value < right_value.real_value;
+                    typed.value.boolean = left_value.value.real < right_value.value.real;
                 } else if (right_value.type == integer_type) {
-                    value.boolean_value = left_value.real_value < right_value.integer_value;
+                    typed.value.boolean = left_value.value.real < right_value.value.integer;
                 } else {
                     filter_log_msg(LOG_LEVEL_ERROR, "Type error in expression evaluation: non-numeric value on right side of <\n");
                 }
             } else if (left_value.type == integer_type) {
                 if (right_value.type == real_type) {
-                    value.boolean_value = left_value.integer_value < right_value.real_value;
+                    typed.value.boolean = left_value.value.integer < right_value.value.real;
                 } else if (right_value.type == integer_type) {
-                    value.boolean_value = left_value.integer_value < right_value.integer_value;
+                    typed.value.boolean = left_value.value.integer < right_value.value.integer;
                 } else {
                     filter_log_msg(LOG_LEVEL_ERROR, "Type error in expression evaluation: non-numeric value on right side of <\n");
                 }
             } else {
                 filter_log_msg(LOG_LEVEL_ERROR, "Type error in expression evaluation: non-numeric value on left side of <\n");
             }
-            push(stack, &value);
+            push(stack, &typed);
             break;
 
         case action_gt:
             pop(stack, &right_value);
             pop(stack, &left_value);
-            value.type = boolean_type;
+            typed.type = boolean_type;
             // default to false if there is an error
-            value.boolean_value = false;
+            typed.value.boolean = false;
 
             if (left_value.type == real_type) {
                 if (right_value.type == real_type) {
-                    value.boolean_value = left_value.real_value > right_value.real_value;
+                    typed.value.boolean = left_value.value.real > right_value.value.real;
                 } else if (right_value.type == integer_type) {
-                    value.boolean_value = left_value.real_value > right_value.integer_value;
+                    typed.value.boolean = left_value.value.real > right_value.value.integer;
                 } else {
                     filter_log_msg(LOG_LEVEL_ERROR, "Type error in expression evaluation: non-numeric value on right side of >\n");
                 }
             } else if (left_value.type == integer_type) {
                 if (right_value.type == real_type) {
-                    value.boolean_value = left_value.integer_value > right_value.real_value;
+                    typed.value.boolean = left_value.value.integer > right_value.value.real;
                 } else if (right_value.type == integer_type) {
-                    value.boolean_value = left_value.integer_value > right_value.integer_value;
+                    typed.value.boolean = left_value.value.integer > right_value.value.integer;
                 } else {
                     filter_log_msg(LOG_LEVEL_ERROR, "Type error in expression evaluation: non-numeric value on right side of >\n");
                 }
             } else {
                 filter_log_msg(LOG_LEVEL_ERROR, "Type error in expression evaluation: non-numeric value on left side of >\n");
             }
-            push(stack, &value);
+            push(stack, &typed);
             break;
 
         case action_lte:
             pop(stack, &right_value);
             pop(stack, &left_value);
-            value.type = boolean_type;
+            typed.type = boolean_type;
             // default to false if there is an error
-            value.boolean_value = false;
+            typed.value.boolean = false;
 
             if (left_value.type == real_type) {
                 if (right_value.type == real_type) {
-                    value.boolean_value = left_value.real_value <= right_value.real_value;
+                    typed.value.boolean = left_value.value.real <= right_value.value.real;
                 } else if (right_value.type == integer_type) {
-                    value.boolean_value = left_value.real_value <= right_value.integer_value;
+                    typed.value.boolean = left_value.value.real <= right_value.value.integer;
                 } else {
                     filter_log_msg(LOG_LEVEL_ERROR, "Type error in expression evaluation: non-numeric value on right side of <=\n");
                 }
             } else if (left_value.type == integer_type) {
                 if (right_value.type == real_type) {
-                    value.boolean_value = left_value.integer_value <= right_value.real_value;
+                    typed.value.boolean = left_value.value.integer <= right_value.value.real;
                 } else if (right_value.type == integer_type) {
-                    value.boolean_value = left_value.integer_value <= right_value.integer_value;
+                    typed.value.boolean = left_value.value.integer <= right_value.value.integer;
                 } else {
                     filter_log_msg(LOG_LEVEL_ERROR, "Type error in expression evaluation: non-numeric value on right side of <=\n");
                 }
             } else {
                 filter_log_msg(LOG_LEVEL_ERROR, "Type error in expression evaluation: non-numeric value on left side of <=\n");
             }
-            push(stack, &value);
+            push(stack, &typed);
             break;
 
         case action_gte:
             pop(stack, &right_value);
             pop(stack, &left_value);
-            value.type = boolean_type;
+            typed.type = boolean_type;
             // default to false if there is an error
-            value.boolean_value = false;
+            typed.value.boolean = false;
 
             if (left_value.type == real_type) {
                 if (right_value.type == real_type) {
-                    value.boolean_value = left_value.real_value >= right_value.real_value;
+                    typed.value.boolean = left_value.value.real >= right_value.value.real;
                 } else if (right_value.type == integer_type) {
-                    value.boolean_value = left_value.real_value >= right_value.integer_value;
+                    typed.value.boolean = left_value.value.real >= right_value.value.integer;
                 } else {
                     filter_log_msg(LOG_LEVEL_ERROR, "Type error in expression evaluation: non-numeric value on right side of >=\n");
                 }
             } else if (left_value.type == integer_type) {
                 if (right_value.type == real_type) {
-                    value.boolean_value = left_value.integer_value >= right_value.real_value;
+                    typed.value.boolean = left_value.value.integer >= right_value.value.real;
                 } else if (right_value.type == integer_type) {
-                    value.boolean_value = left_value.integer_value >= right_value.integer_value;
+                    typed.value.boolean = left_value.value.integer >= right_value.value.integer;
                 } else {
                     filter_log_msg(LOG_LEVEL_ERROR, "Type error in expression evaluation: non-numeric value on right side of >=\n");
                 }
             } else {
                 filter_log_msg(LOG_LEVEL_ERROR, "Type error in expression evaluation: non-numeric value on left side of >=\n");
             }
-            push(stack, &value);
+            push(stack, &typed);
             break;
 
         case action_eq:
             pop(stack, &right_value);
             pop(stack, &left_value);
-            value.type = boolean_type;
+            typed.type = boolean_type;
             // default to false if there is an error
-            value.boolean_value = false;
+            typed.value.boolean = false;
 
             if (left_value.type == real_type) {
                 if (right_value.type == real_type) {
-                    value.boolean_value = left_value.real_value == right_value.real_value;
+                    typed.value.boolean = left_value.value.real == right_value.value.real;
                 } else if (right_value.type == integer_type) {
-                    value.boolean_value = left_value.real_value == right_value.integer_value;
+                    typed.value.boolean = left_value.value.real == right_value.value.integer;
                 } else {
                     filter_log_msg(LOG_LEVEL_ERROR, "Type error in expression evaluation: non-numeric value on right side of = but numeric on left\n");
                 }
             } else if (left_value.type == integer_type) {
                 if (right_value.type == real_type) {
-                    value.boolean_value = left_value.integer_value == right_value.real_value;
+                    typed.value.boolean = left_value.value.integer == right_value.value.real;
                 } else if (right_value.type == integer_type) {
-                    value.boolean_value = left_value.integer_value == right_value.integer_value;
+                    typed.value.boolean = left_value.value.integer == right_value.value.integer;
                 } else {
                     filter_log_msg(LOG_LEVEL_ERROR, "Type error in expression evaluation: non-numeric value on right side of = but numeric on left\n");
                 }
             } else if (left_value.type == string_type) {
                 if (right_value.type == string_type) {
-                    value.boolean_value = left_value.string_value == right_value.string_value;
+                    typed.value.boolean = left_value.value.string == right_value.value.string;
                 } else {
                     filter_log_msg(LOG_LEVEL_ERROR, "Type error in expression evaluation: non-string value on right side of = but string on left\n");
                 }
             } else if (left_value.type == boolean_type) {
                 if (right_value.type == boolean_type) {
-                    value.boolean_value = left_value.boolean_value == right_value.boolean_value;
+                    typed.value.boolean = left_value.value.boolean == right_value.value.boolean;
                 } else {
                     filter_log_msg(LOG_LEVEL_ERROR, "Type error in expression evaluation: non-Boolean value on right side of = but Boolean on left\n");
                 }
@@ -379,41 +379,41 @@ void evaluate_expression(expression_input *input, typed_value *result, data_stac
                 // it isn't possible to get a pointer type
                 filter_log_msg(LOG_LEVEL_ERROR, "Type error in expression evaluation: NULL on left side of =\n");
             }
-            push(stack, &value);
+            push(stack, &typed);
             break;
 
         case action_ne:
             pop(stack, &right_value);
             pop(stack, &left_value);
-            value.type = boolean_type;
+            typed.type = boolean_type;
             // default to false if there is an error
-            value.boolean_value = false;
+            typed.value.boolean = false;
 
             if (left_value.type == real_type) {
                 if (right_value.type == real_type) {
-                    value.boolean_value = left_value.real_value != right_value.real_value;
+                    typed.value.boolean = left_value.value.real != right_value.value.real;
                 } else if (right_value.type == integer_type) {
-                    value.boolean_value = left_value.real_value != right_value.integer_value;
+                    typed.value.boolean = left_value.value.real != right_value.value.integer;
                 } else {
                     filter_log_msg(LOG_LEVEL_ERROR, "Type error in expression evaluation: non-numeric value on right side of != but numeric on left\n");
                 }
             } else if (left_value.type == integer_type) {
                 if (right_value.type == real_type) {
-                    value.boolean_value = left_value.integer_value != right_value.real_value;
+                    typed.value.boolean = left_value.value.integer != right_value.value.real;
                 } else if (right_value.type == integer_type) {
-                    value.boolean_value = left_value.integer_value != right_value.integer_value;
+                    typed.value.boolean = left_value.value.integer != right_value.value.integer;
                 } else {
                     filter_log_msg(LOG_LEVEL_ERROR, "Type error in expression evaluation: non-numeric value on right side of != but numeric on left\n");
                 }
             } else if (left_value.type == string_type) {
                 if (right_value.type == string_type) {
-                    value.boolean_value = left_value.string_value != right_value.string_value;
+                    typed.value.boolean = left_value.value.string != right_value.value.string;
                 } else {
                     filter_log_msg(LOG_LEVEL_ERROR, "Type error in expression evaluation: non-string value on right side of != but string on left\n");
                 }
             } else if (left_value.type == boolean_type) {
                 if (right_value.type == boolean_type) {
-                    value.boolean_value = left_value.boolean_value != right_value.boolean_value;
+                    typed.value.boolean = left_value.value.boolean != right_value.value.boolean;
                 } else {
                     filter_log_msg(LOG_LEVEL_ERROR, "Type error in expression evaluation: non-Boolean value on right side of = but Boolean on left\n");
                 }
@@ -421,133 +421,133 @@ void evaluate_expression(expression_input *input, typed_value *result, data_stac
                 // it isn't possible to get a pointer type
                 filter_log_msg(LOG_LEVEL_ERROR, "Type error in expression evaluation: NULL on left side of !=\n");
             }
-            push(stack, &value);
+            push(stack, &typed);
             break;
 
         case action_and:
             pop(stack, &right_value);
             pop(stack, &left_value);
-            value.type = boolean_type;
+            typed.type = boolean_type;
             // default to false if there is an error
-            value.boolean_value = false;
+            typed.value.boolean = false;
 
             if (left_value.type == boolean_type) {
                 if (right_value.type == boolean_type) {
-                    value.boolean_value = left_value.boolean_value && right_value.boolean_value;
+                    typed.value.boolean = left_value.value.boolean && right_value.value.boolean;
                 } else {
                     filter_log_msg(LOG_LEVEL_ERROR, "Type error in expression evaluation: non-Boolean value on right side of &\n");
                 }
             } else {
                 filter_log_msg(LOG_LEVEL_ERROR, "Type error in expression evaluation: non-Boolean value on left side of &\n");
             }
-            push(stack, &value);
+            push(stack, &typed);
             break;
 
         case action_or:
             pop(stack, &right_value);
             pop(stack, &left_value);
-            value.type = boolean_type;
+            typed.type = boolean_type;
             // default to false if there is an error
-            value.boolean_value = false;
+            typed.value.boolean = false;
 
             if (left_value.type == boolean_type) {
                 if (right_value.type == boolean_type) {
-                    value.boolean_value = left_value.boolean_value || right_value.boolean_value;
+                    typed.value.boolean = left_value.value.boolean || right_value.value.boolean;
                 } else {
                     filter_log_msg(LOG_LEVEL_ERROR, "Type error in expression evaluation: non-Boolean value on right side of |\n");
                 }
             } else {
                 filter_log_msg(LOG_LEVEL_ERROR, "Type error in expression evaluation: non-Boolean value on left side of |\n");
             }
-            push(stack, &value);
+            push(stack, &typed);
             break;
 
         case action_neg:
             pop(stack, &left_value);
 
             if (left_value.type == integer_type) {
-                value.type = integer_type;
-                value.integer_value = -left_value.integer_value;
+                typed.type = integer_type;
+                typed.value.integer = -left_value.value.integer;
             } else if (left_value.type == real_type) {
-                value.type = real_type;
-                value.real_value = -left_value.real_value;
+                typed.type = real_type;
+                typed.value.real = -left_value.value.real;
             } else {
                 filter_log_msg(LOG_LEVEL_ERROR, "Type error in expression evaluation: non-numeric value modified by unary -\n");
-                value.type = integer_type;
-                value.integer_value = 0;
+                typed.type = integer_type;
+                typed.value.integer = 0;
             }
-            push(stack, &value);
+            push(stack, &typed);
             break;
 
         case action_not:
             pop(stack, &left_value);
-            value.type = boolean_type;
+            typed.type = boolean_type;
             // default to false if there is an error
-            value.boolean_value = false;
+            typed.value.boolean = false;
 
             if (left_value.type == boolean_type) {
-                value.boolean_value = !left_value.boolean_value;
+                typed.value.boolean = !left_value.value.boolean;
             } else {
                 filter_log_msg(LOG_LEVEL_ERROR, "Type error in expression evaluation: non-Boolean value modified by unary !\n");
             }
-            push(stack, &value);
+            push(stack, &typed);
             break;
 
         case param_boollit:
-            value.type = boolean_type;
-            value.boolean_value = input[position++].boolean_value;
-            push(stack, &value);
+            typed.type = boolean_type;
+            typed.value.boolean = input[position++].boolean_value;
+            push(stack, &typed);
             break;
         case param_intlit:
-            value.type = integer_type;
-            value.integer_value = input[position++].integer_value;
-            push(stack, &value);
+            typed.type = integer_type;
+            typed.value.integer = input[position++].integer_value;
+            push(stack, &typed);
             break;
         case param_reallit:
-            value.type = real_type;
-            value.real_value = input[position++].real_value;
-            push(stack, &value);
+            typed.type = real_type;
+            typed.value.real = input[position++].real_value;
+            push(stack, &typed);
             break;
         case param_strlit:
-            value.type = string_type;
-            value.string_value = input[position++].string_value;
-            push(stack, &value);
+            typed.type = string_type;
+            typed.value.string = input[position++].string_value;
+            push(stack, &typed);
             break;
         case param_left_field:
             key = input[position++].field_name;
-            map_get(left_map, key, &value);
-            if (value.type == null_type) {
+            map_get(left_map, key, &typed);
+            if (typed.type == null_type) {
                 filter_log_msg(LOG_LEVEL_WARN, "Type error in expression evaluation: Null type for %d\n", key);
             }
-            push(stack, &value);
+            push(stack, &typed);
             break;
         case param_right_field:
             key = input[position++].field_name;
-            map_get(right_map, key, &value);
-            if (value.type == null_type) {
+            map_get(right_map, key, &typed);
+            if (typed.type == null_type) {
                 filter_log_msg(LOG_LEVEL_WARN, "Type error in expression evaluation: Null type for %d\n", key);
             }
-            push(stack, &value);
+            push(stack, &typed);
             break;
         case param_left_begin:
-            value.type = integer_type;
-            value.integer_value = left_begin;
-            push(stack, &value);
+            typed.type = integer_type;
+            typed.value.integer = left_begin;
+            push(stack, &typed);
             break;
         case param_left_end:
-            value.type = integer_type;
-            value.integer_value = left_end;
-            push(stack, &value);
+            typed.type = integer_type;
+            typed.value.integer = left_end;
+            push(stack, &typed);
             break;
         case param_right_begin:
-            value.type = integer_type;
-            value.integer_value = right_begin;
-            push(stack, &value);
+            typed.type = integer_type;
+            typed.value.integer = right_begin;
+            push(stack, &typed);
             break;
         case param_right_end:
-            value.type = integer_type;
-            value.integer_value = right_end;
-            push(stack, &value);
+            typed.type = integer_type;
+            typed.value.integer = right_end;
+            push(stack, &typed);
             break;
         }
 
@@ -617,13 +617,13 @@ unsigned int max_expression_stack_depth(expression_input *input) {
     return max;
 }
 
+#ifndef NO_DYNAMIC_MEMORY
 void write_expression(expression_input *input,
                       dictionary *key_dict,
                       dictionary *val_dict,
                       const char *left_name,
                       const char *right_name,
                       int log_to) {
-#ifndef NO_DYNAMIC_MEMORY
     char *begin = "begin", *end = "end", *true_str = "true", *false_str = "false";
     unsigned int position, length;
     expression_action action;
@@ -933,15 +933,17 @@ void write_expression(expression_input *input,
         stack[tos++].str  = buffer.str;
     }
     // actually print out the result
-    tos--;
-    write_msg(log_to, "%.*s", stack[tos].length, stack[tos].str);
+    if (tos > 0) {
+        tos--;
+        write_msg(log_to, "%.*s", stack[tos].length, stack[tos].str);
 
-    // make sure the stack is empty
-    while (tos > 0) {
-        free(stack[--tos].str);
+        // make sure the stack is empty
+        while (tos > 0) {
+            free(stack[--tos].str);
+        }
     }
+
     // free the stack
     free(stack);
-#endif
 }
-
+#endif
\ No newline at end of file
