@@ -83,38 +83,38 @@ operator_code get_operator_from_token(int token) {
 /**
  * Returns true if this expr_node or any of its children belong in the passed BIE node
  */
-static bool belongs_in_bie(ast_node *bie_node, ast_node *expr_node) {
-    if (!bie_node || !expr_node) {
+static bool belongs_in_ie(ast_node *ie_node, ast_node *expr_node) {
+    if (!ie_node || !expr_node) {
         return false;
     }
 
     switch(expr_node->type) {
     case type_unary_expr:
-        return belongs_in_bie(bie_node, expr_node->unary_expr.operand);
+        return belongs_in_ie(ie_node, expr_node->unary_expr.operand);
         break;
     case type_binary_expr:
         /* if this is a Boolean type operator, we want to recurse */
         if (expr_node->binary_expr.operator == AND || expr_node->binary_expr.operator == OR) {
-            return belongs_in_bie(bie_node, expr_node->binary_expr.left) ||
-                    belongs_in_bie(bie_node, expr_node->binary_expr.right);
+            return belongs_in_ie(ie_node, expr_node->binary_expr.left) ||
+                    belongs_in_ie(ie_node, expr_node->binary_expr.right);
         } else {
             /* otherwise, just check if the belongs_in interval expression matches */
-            return expr_node->binary_expr.belongs_in == bie_node;
+            return expr_node->binary_expr.belongs_in == ie_node;
         }
 
         break;
     case type_map_field:
-        return expr_node->map_field.interval_expression == bie_node;
+        return expr_node->map_field.interval_expression == ie_node;
         break;
     case type_time_field:
-        return expr_node->time_field.interval_expression == bie_node;
+        return expr_node->time_field.interval_expression == ie_node;
         break;
     default:
         return true;
     }
 }
 
-static unsigned int generate_eval_from_expr(ast_node *bie_node, ast_node *node, expression_input *input, unsigned int input_position) {
+static unsigned int generate_eval_from_expr(ast_node *ie_node, ast_node *node, expression_input *input, unsigned int input_position) {
     unsigned int added = 0;
     if (!node) {
         return 0;
@@ -141,7 +141,7 @@ static unsigned int generate_eval_from_expr(ast_node *bie_node, ast_node *node,
         added = 2;
         break;
     case type_unary_expr:
-        added = generate_eval_from_expr(bie_node, node->unary_expr.operand, input, input_position);
+        added = generate_eval_from_expr(ie_node, node->unary_expr.operand, input, input_position);
         switch(node->unary_expr.operator) {
         case UMINUS:
             input[input_position + added].action = action_neg;
@@ -155,6 +155,7 @@ static unsigned int generate_eval_from_expr(ast_node *bie_node, ast_node *node,
     case type_binary_expr:
         /**
          * An expression node may be affiliated with a BIE, multiple BIEs or it may be neutral.
+         * (also, now it may be affiliated witn an AIE, but only in special circumstances).
          * For children of binary Boolean operators (&, |), the affiliation of the children determine
          * whether or not they are included in nested rules.
          *
@@ -192,7 +193,7 @@ static unsigned int generate_eval_from_expr(ast_node *bie_node, ast_node *node,
         if (node->binary_expr.operator == AND || node->binary_expr.operator == OR) {
             filter_log_msg(LOG_LEVEL_DEBUG, "      Generate eval: found Boolean binary expr\n");
             // if the left side doesn't reference the BIE
-            if (!belongs_in_bie(bie_node, node->binary_expr.left)) {
+            if (!belongs_in_ie(ie_node, node->binary_expr.left)) {
                 filter_log_msg(LOG_LEVEL_DEBUG, "      Generate eval: left doesn't belong in BIE\n");
                 // not referenced on the left side! so just write a Boolean
                 input[input_position].action = param_boollit;
@@ -208,11 +209,11 @@ static unsigned int generate_eval_from_expr(ast_node *bie_node, ast_node *node,
                 added = 2;
             } else {
                 filter_log_msg(LOG_LEVEL_DEBUG, "      Generate eval: left belongs in BIE\n");
-                added = generate_eval_from_expr(bie_node, node->binary_expr.left, input, input_position);
+                added = generate_eval_from_expr(ie_node, node->binary_expr.left, input, input_position);
             }
 
             // if the right side doesn't reference the BIE
-            if (!belongs_in_bie(bie_node, node->binary_expr.right)) {
+            if (!belongs_in_ie(ie_node, node->binary_expr.right)) {
                 filter_log_msg(LOG_LEVEL_DEBUG, "      Generate eval: right doesn't belong in BIE\n");
                 // not referenced on the right side! so just write a Boolean
                 input[input_position + added].action = param_boollit;
@@ -228,12 +229,12 @@ static unsigned int generate_eval_from_expr(ast_node *bie_node, ast_node *node,
                 added += 2;
             } else {
                 filter_log_msg(LOG_LEVEL_DEBUG, "      Generate eval: right belongs in BIE\n");
-                added += generate_eval_from_expr(bie_node, node->binary_expr.right, input, input_position + added);
+                added += generate_eval_from_expr(ie_node, node->binary_expr.right, input, input_position + added);
             }
         } else {
             // not a Boolean binary expression
-            added = generate_eval_from_expr(bie_node, node->binary_expr.left, input, input_position);
-            added += generate_eval_from_expr(bie_node, node->binary_expr.right, input, input_position + added);
+            added = generate_eval_from_expr(ie_node, node->binary_expr.left, input, input_position);
+            added += generate_eval_from_expr(ie_node, node->binary_expr.right, input, input_position + added);
         }
 
         switch(node->binary_expr.operator) {
@@ -295,7 +296,7 @@ static unsigned int generate_eval_from_expr(ast_node *bie_node, ast_node *node,
         added = 2;
         break;
     case type_time_field:
-        if (node->time_field.is_time || node->time_field.interval_expression == bie_node) {
+        if (node->time_field.is_time || node->time_field.interval_expression == ie_node) {
             switch(node->time_field.side) {
             case left_side:
                 switch(node->time_field.time_field) {
@@ -338,7 +339,7 @@ static unsigned int generate_eval_from_expr(ast_node *bie_node, ast_node *node,
     return added;
 }
 
-static unsigned int get_eval_size(ast_node *bie_node, ast_node *node) {
+static unsigned int get_eval_size(ast_node *ie_node, ast_node *node) {
     unsigned int size = 0;
     if (!node) {
         return 0;
@@ -357,29 +358,29 @@ static unsigned int get_eval_size(ast_node *bie_node, ast_node *node) {
         size = 2;
         break;
     case type_unary_expr:
-        size = 1 + get_eval_size(bie_node, node->unary_expr.operand);
+        size = 1 + get_eval_size(ie_node, node->unary_expr.operand);
         break;
     case type_binary_expr:
         if (node->binary_expr.operator == AND || node->binary_expr.operator == OR) {
             // if the left side doesn't reference the BIE
-            if (!belongs_in_bie(bie_node, node->binary_expr.left)) {
+            if (!belongs_in_ie(ie_node, node->binary_expr.left)) {
                 // not referenced on the left side!
                 size = 2;
             } else {
-                size = get_eval_size(bie_node, node->binary_expr.left);
+                size = get_eval_size(ie_node, node->binary_expr.left);
             }
 
             // if the right side doesn't reference the BIE
-            if (!belongs_in_bie(bie_node, node->binary_expr.right)) {
+            if (!belongs_in_ie(ie_node, node->binary_expr.right)) {
                 // not referenced on the right side!
                 size += 2;
             } else {
-                size += get_eval_size(bie_node, node->binary_expr.right);
+                size += get_eval_size(ie_node, node->binary_expr.right);
             }
         } else {
             // not a Boolean binary expression
-            size = get_eval_size(bie_node, node->binary_expr.left);
-            size += get_eval_size(bie_node, node->binary_expr.right);
+            size = get_eval_size(ie_node, node->binary_expr.left);
+            size += get_eval_size(ie_node, node->binary_expr.right);
         }
         size += 1;
         break;
@@ -401,12 +402,12 @@ static unsigned int get_eval_size(ast_node *bie_node, ast_node *node) {
     return size;
 }
 
-static expression_input * generate_eval_from_map_field(ast_node *bie_node, map_value *value, side_enum side) {
+static expression_input * generate_eval_from_map_field(ast_node *ie_node, map_value *field, side_enum side) {
     expression_input *expr = NULL;
     int size;
     ast_node *node;
 
-    if (value->type == string_type) {
+    if (field->type == string_type) {
         initialize_expression_input(&expr, 3);
         expr[0].length = 3;
         switch(side) {
@@ -417,18 +418,18 @@ static expression_input * generate_eval_from_map_field(ast_node *bie_node, map_v
             expr[1].action = param_right_field;
             break;
         }
-        expr[2].string_value = value->string_value;
-    } else if (value->type == pointer_type) {
-        node = (ast_node *)value->pointer_value;
-        size = 1 + get_eval_size(bie_node, node);
+        expr[2].string_value = field->value.string;
+    } else if (field->type == pointer_type) {
+        node = (ast_node *)field->value.pointer;
+        size = 1 + get_eval_size(ie_node, node);
         initialize_expression_input(&expr, size);
         expr[0].length = size;
-        generate_eval_from_expr(bie_node, node, expr, 1);
-    } else if (value->type == boolean_type) {
+        generate_eval_from_expr(ie_node, node, expr, 1);
+    } else if (field->type == boolean_type) {
         initialize_expression_input(&expr, 3);
         expr[0].length = 3;
         expr[1].action = param_boollit;
-        expr[2].boolean_value = value->boolean_value;
+        expr[2].boolean_value = field->value.boolean;
     }
 
     return expr;
@@ -443,28 +444,28 @@ static void generate_evals_from_time_maps(data_map *map, word_id left_begin, wor
         initialize_expression_input(&expression, 2);
         expression[0].length = 2;
         expression[1].action = param_left_begin;
-        value.pointer_value = expression;
+        value.value.pointer = expression;
         map_set(map, left_begin, &value);
     }
     if (left_end != WORD_NOT_FOUND) {
         initialize_expression_input(&expression, 2);
         expression[0].length = 2;
         expression[1].action = param_left_end;
-        value.pointer_value = expression;
+        value.value.pointer = expression;
         map_set(map, left_end, &value);
     }
     if (right_begin != WORD_NOT_FOUND) {
         initialize_expression_input(&expression, 2);
         expression[0].length = 2;
         expression[1].action = param_right_begin;
-        value.pointer_value = expression;
+        value.value.pointer = expression;
         map_set(map, right_begin, &value);
     }
     if (right_end != WORD_NOT_FOUND) {
         initialize_expression_input(&expression, 2);
         expression[0].length = 2;
         expression[1].action = param_right_end;
-        value.pointer_value = expression;
+        value.value.pointer = expression;
         map_set(map, right_end, &value);
     }
 }
@@ -478,61 +479,92 @@ static nfer_rule * generate_each_rule(ast_node *node, nfer_specification *spec,
 
     if (!node) {
         rule = NULL;
-    }
-    switch (node->type) {
-    case type_atomic_interval_expr:
-        rule = NULL;
-        break;
-    case type_binary_interval_expr:
-        generate_each_rule(node->binary_interval_expr.left, spec, node->binary_interval_expr.left_name, where_node);
-        generate_each_rule(node->binary_interval_expr.right, spec, node->binary_interval_expr.right_name, where_node);
+    } else {
+        switch (node->type) {
+        case type_atomic_interval_expr:
+            // check to see if we have to generate a nested atomic rule
+            if (node->atomic_interval_expr.separate) {
+                // there's no nesting below this, so don't worry about that
+                rule = add_rule_to_specification(spec, result, node->atomic_interval_expr.result_id, ALSO_OPERATOR, WORD_NOT_FOUND, NULL);
+                // always hidden!
+                rule->hidden = true;
+                initialize_map(&rule->map_expressions);
 
-        rule = add_rule_to_specification(spec, result, node->binary_interval_expr.left_name,
-                    get_operator_from_token(node->binary_interval_expr.interval_op),
-                    node->binary_interval_expr.right_name, NULL);
-        // default to hidden, but the top level rule will set this back to false
-        rule->hidden = true;
-        initialize_map(&rule->map_expressions);
+                if (where_node) {
+                    if (belongs_in_ie(node, where_node)) {
+                        size = 1 + get_eval_size(node, where_node);
+                        initialize_expression_input(&rule->where_expression, size);
+                        rule->where_expression[0].length = size;
+                        filter_log_msg(LOG_LEVEL_DEBUG, "    Generating eval for AIE where clause, eval size is %d\n", size);
+                        generate_eval_from_expr(node, where_node, rule->where_expression, 1);
+                    }
+                }
+                get_map_iterator(&node->atomic_interval_expr.field_map, &mit);
+                while (has_next_map_key(&mit)) {
+                    key = next_map_key(&mit);
+                    map_get(&node->atomic_interval_expr.field_map, key, &value);
 
-        if (where_node) {
-            // verify that there is anything in the where expression that should occur in this rule
-            if (belongs_in_bie(node, where_node)) {
-                size = 1 + get_eval_size(node, where_node);
-                initialize_expression_input(&rule->where_expression, size);
-                rule->where_expression[0].length = size;
-                filter_log_msg(LOG_LEVEL_DEBUG, "    Generating eval for BIE where clause, eval size is %d\n", size);
-                generate_eval_from_expr(node, where_node, rule->where_expression, 1);
+                    expression.type = pointer_type;
+                    expression.value.pointer = generate_eval_from_map_field(node, &value, left_side);
+                    map_set(&rule->map_expressions, key, &expression);
+                }
+                generate_evals_from_time_maps(&rule->map_expressions, node->atomic_interval_expr.begin_map, node->atomic_interval_expr.end_map,
+                    WORD_NOT_FOUND, WORD_NOT_FOUND);
+            } else {
+                rule = NULL;
             }
-        }
-        get_map_iterator(&node->binary_interval_expr.left_field_map, &mit);
-        while (has_next_map_key(&mit)) {
-            key = next_map_key(&mit);
-            map_get(&node->binary_interval_expr.left_field_map, key, &value);
+            break;
+        case type_binary_interval_expr:
+            generate_each_rule(node->binary_interval_expr.left, spec, node->binary_interval_expr.left_name, where_node);
+            generate_each_rule(node->binary_interval_expr.right, spec, node->binary_interval_expr.right_name, where_node);
 
-            expression.type = pointer_type;
-            expression.pointer_value = generate_eval_from_map_field(node, &value, left_side);
-            map_set(&rule->map_expressions, key, &expression);
-        }
-        get_map_iterator(&node->binary_interval_expr.right_field_map, &mit);
-        while (has_next_map_key(&mit)) {
-            key = next_map_key(&mit);
-            map_get(&node->binary_interval_expr.right_field_map, key, &value);
+            rule = add_rule_to_specification(spec, result, node->binary_interval_expr.left_name,
+                        get_operator_from_token(node->binary_interval_expr.interval_op),
+                        node->binary_interval_expr.right_name, NULL);
+            // default to hidden, but the top level rule will set this back to false
+            rule->hidden = true;
+            initialize_map(&rule->map_expressions);
 
-            expression.type = pointer_type;
-            expression.pointer_value = generate_eval_from_map_field(node, &value, right_side);
-            map_set(&rule->map_expressions, key, &expression);
-        }
-        generate_evals_from_time_maps(&rule->map_expressions, node->binary_interval_expr.left_begin_map, node->binary_interval_expr.left_end_map,
-                node->binary_interval_expr.right_begin_map, node->binary_interval_expr.right_end_map);
+            if (where_node) {
+                // verify that there is anything in the where expression that should occur in this rule
+                if (belongs_in_ie(node, where_node)) {
+                    size = 1 + get_eval_size(node, where_node);
+                    initialize_expression_input(&rule->where_expression, size);
+                    rule->where_expression[0].length = size;
+                    filter_log_msg(LOG_LEVEL_DEBUG, "    Generating eval for BIE where clause, eval size is %d\n", size);
+                    generate_eval_from_expr(node, where_node, rule->where_expression, 1);
+                }
+            }
+            get_map_iterator(&node->binary_interval_expr.left_field_map, &mit);
+            while (has_next_map_key(&mit)) {
+                key = next_map_key(&mit);
+                map_get(&node->binary_interval_expr.left_field_map, key, &value);
 
-        break;
-    default:
-        rule = NULL;
+                expression.type = pointer_type;
+                expression.value.pointer = generate_eval_from_map_field(node, &value, left_side);
+                map_set(&rule->map_expressions, key, &expression);
+            }
+            get_map_iterator(&node->binary_interval_expr.right_field_map, &mit);
+            while (has_next_map_key(&mit)) {
+                key = next_map_key(&mit);
+                map_get(&node->binary_interval_expr.right_field_map, key, &value);
+
+                expression.type = pointer_type;
+                expression.value.pointer = generate_eval_from_map_field(node, &value, right_side);
+                map_set(&rule->map_expressions, key, &expression);
+            }
+            generate_evals_from_time_maps(&rule->map_expressions, node->binary_interval_expr.left_begin_map, node->binary_interval_expr.left_end_map,
+                    node->binary_interval_expr.right_begin_map, node->binary_interval_expr.right_end_map);
+
+            break;
+        default:
+            rule = NULL;
+        }
     }
     return rule;
 }
 
-static void generate_eval_from_map_expr_list(ast_node *bie_node, ast_node *map_expr_list, data_map *map) {
+static void generate_eval_from_map_expr_list(ast_node *ie_node, ast_node *map_expr_list, data_map *map) {
     expression_input *expression;
     map_value value;
     int size;
@@ -540,15 +572,15 @@ static void generate_eval_from_map_expr_list(ast_node *bie_node, ast_node *map_e
         return;
     }
 
-    size = 1 + get_eval_size(bie_node, map_expr_list->map_expr_list.map_expr);
+    size = 1 + get_eval_size(ie_node, map_expr_list->map_expr_list.map_expr);
     initialize_expression_input(&expression, size);
     expression[0].length = size;
-    generate_eval_from_expr(bie_node, map_expr_list->map_expr_list.map_expr, expression, 1);
+    generate_eval_from_expr(ie_node, map_expr_list->map_expr_list.map_expr, expression, 1);
     value.type = pointer_type;
-    value.pointer_value = expression;
+    value.value.pointer = expression;
     map_set(map, map_expr_list->map_expr_list.resulting_map_key, &value);
 
-    generate_eval_from_map_expr_list(bie_node, map_expr_list->map_expr_list.tail, map);
+    generate_eval_from_map_expr_list(ie_node, map_expr_list->map_expr_list.tail, map);
 }
 
 bool in_imports(ast_node *imports, word_id name) {
