@@ -140,20 +140,20 @@ static PyObject * map_to_pyobject(data_map *map) {
             py_dict_value = NULL;
             break;
         case boolean_type:
-            if (value.boolean_value) {
+            if (value.value.boolean) {
                 py_dict_value = Py_True;
             } else {
                 py_dict_value = Py_False;
             }
             break;
         case integer_type:
-            py_dict_value = PyLong_FromUnsignedLongLong(value.integer_value);
+            py_dict_value = PyLong_FromUnsignedLongLong(value.value.integer);
             break;
         case real_type:
-            py_dict_value = PyFloat_FromDouble(value.real_value);
+            py_dict_value = PyFloat_FromDouble(value.value.real);
             break;
         case string_type:
-            py_dict_value = PyUnicode_FromString(get_word(&val_dict, value.string_value));
+            py_dict_value = PyUnicode_FromString(get_word(&val_dict, value.value.string));
             break;
         }
 
@@ -195,10 +195,9 @@ static PyObject * pool_to_pyobject(pool *p) {
 }
 
 static bool pyobject_to_map(PyObject *py_dict, data_map *map) {
-    const char *key_string;
+    const char *key_string, *string_value;
     map_key key;
     map_value value;
-    char *string_value;
     label string_label;
 
     PyObject *py_dict_key, *py_dict_value;
@@ -223,19 +222,19 @@ static bool pyobject_to_map(PyObject *py_dict, data_map *map) {
         // this is tricky because we are going between two loosely typed languages
         // we have to check for bool first, because it also counts as a long!
         if (PyBool_Check(py_dict_value)) {
-            value.boolean_value = (py_dict_value == Py_True);
+            value.value.boolean = (py_dict_value == Py_True);
             value.type = boolean_type;
         } else if (PyLong_Check(py_dict_value)) {
-            value.integer_value = PyLong_AsUnsignedLongLong(py_dict_value);
+            value.value.integer = PyLong_AsUnsignedLongLong(py_dict_value);
             value.type = integer_type;
         } else if (PyFloat_Check(py_dict_value)) {
-            value.real_value = PyFloat_AsDouble(py_dict_value);
+            value.value.real = PyFloat_AsDouble(py_dict_value);
             value.type = real_type;
         } else if (PyUnicode_Check(py_dict_value)) {
             string_value = PyUnicode_AsUTF8AndSize(py_dict_value, &string_size);
             if (string_value != NULL) {
                 string_label = add_word(&val_dict, string_value);
-                value.string_value = string_label;
+                value.value.string = string_label;
                 value.type = string_type;
             }
         } else {
@@ -412,7 +411,7 @@ static PyObject * python_get_publishes(PyObject *Py_UNUSED(self), PyObject *Py_U
     while(has_next_word(&dit)) {
         name = next_word(&dit);
 
-        if (does_publish(&spec, name)) {
+        if (is_published(&spec, name)) {
             python_string = PyUnicode_FromString(get_word(&name_dict, name));
             if (PyList_Append(python_list, python_string) == -1) {
                 // Error!  It sets an error message.
@@ -451,9 +450,9 @@ static PyObject * python_minimal(PyObject *Py_UNUSED(self), PyObject *args) {
 }
 
 static PyMethodDef nfer_methods[] = {
-	{"init",       python_init, METH_NOARGS, init_method_doc},
+    {"init",       python_init, METH_NOARGS, init_method_doc},
     {"load",       python_load_spec, METH_VARARGS, load_method_doc},
-	{"scan",       python_scan_spec, METH_VARARGS, scan_method_doc},
+    {"scan",       python_scan_spec, METH_VARARGS, scan_method_doc},
     {"save",       python_save_spec, METH_VARARGS, save_method_doc},
     {"add",        python_add_interval, METH_VARARGS, add_method_doc},
     {"learn",      python_learn_spec, METH_VARARGS, learn_method_doc},
